Puppet::Type.type(:chattr).provide(:base, parent: Puppet::Provider) do
  desc 'Basic provider for chattr.'
  defaultfor kernel: 'Linux'

  commands lsattr: 'lsattr'
  commands chattr: 'chattr'

  # This creates a bunch of getters/setters for our properties/parameters
  # this is only for prefetch/flush providers
  mk_resource_methods

  def self.properties
    {
      no_atime:        'A',
      append_only:     'a',
      compressed:      'c',
      dirsync:         'D',
      no_dump:         'd',
      immutable:       'i',
      journaling:      'j',
      secure_deletion: 's',
      sync:            'S',
      top_directory:   'T',
      no_tail_merging: 't',
      undeletable:     'u',
    }
  end

  def self.prefetch(resources)
    resources.each do |_, resource|
      output = lsattr(resource[:file])
      is_flags = output.split(' ').first

      result = Hash[
        properties.map do |property, flag|
          [
            property,
            (is_flags.match(flag) ? :true : :false),
          ]
        end
      ]

      resource.provider = new(result)
    end
  end

  def flush
    file  = resource.parameters[:file].value
    props = self.class.properties

    grouped = @property_hash.group_by { |_, value| value }
    disable = (grouped[:false] || []).map { |key, _| props[key] }
    enable  = (grouped[:true ] || []).map { |key, _| props[key] }

    # handle immutable in a seperate call
    immutable = enable.delete('i')

    # disable immutable for this operation
    chattr(['-i', file])

    # disable any flags that should be
    mode = "-#{disable.join}"
    chattr([mode, file])

    # enable any flags that should be
    mode = "+#{enable.join}"
    chattr([mode, file])

    @property_hash.clear
  ensure
    # enable immutable if it should be
    chattr(['+i', file]) if immutable
  end
end
