Puppet::Type.newtype(:chattr) do
  @doc = 'change file attributes on a Linux file system'

  newparam(:file, namevar: true) do
    desc 'file to operate on'
  end

  [
    [ :no_atime,        'When a file with the A attribute set is accessed, its atime record is not modified.' ],
    [ :append_only,     'A file with the a attribute set can only be open in append mode for writing.' ],
    [ :compressed,      'A file with the c attribute set is automatically compressed on the disk by the kernel.' ],
    [ :dirsync,         'When a directory with the D attribute set is modified, the changes are written synchronously on the disk.' ],
    [ :no_dump,         'A file with the d attribute set is not candidate for backup when the dump program is run.' ],
    [ :immutable,       'A file with the i attribute cannot be modified.' ],
    [ :journaling,      'A file with the j attribute has all of its data written to the ext3 journal before being written to the file itself.' ],
    [ :secure_deletion, 'When a file with the s attribute set is deleted, its blocks are zeroed and written back to the disk.' ],
    [ :sync,            'When a file with the S attribute set is modified, the changes are written synchronously on the disk.' ],
    [ :top_directory,   'A directory with the T attribute will be deemed to be the top of directory hierarchies for the purposes of the Orlov block allocator.' ],
    [ :no_tail_merging, 'A file with the t attribute will not have a partial block fragment at the end of the file merged with other files.' ],
    [ :undeletable,     'When a file with the u attribute set is deleted, its contents are saved.' ],
  ].each do |(short_name, long_description)|
    newproperty(short_name, boolean: true) do
      newvalue  :true
      newvalue  :false
      defaultto :false
      desc long_description
      munge { |value| @resource.munge_boolean(value) }
    end
  end

  autorequire(:file) do
    self[:file]
  end

  def munge_boolean(value)
    case value
    when true,  'true',  :true
      :true
    when false, 'false', :false
      :false
    else
      raise 'munge_boolean only takes booleans'
    end
  end
end
